/***************************************************************************
                    hw.c  -  contains basic routines for
                   communicating with the lm_sensors API.
                             -------------------
    begin                : Thu Dec 13 2001
    copyright            : (C) 2001 by Kelledin
    email                : kelledin@users.sourceforge.net
 ***************************************************************************/

 /***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of version 2 of the GNU General Public License as  *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 ***************************************************************************/

#include "mondo.h"

mondo_sensor  *list_start=NULL;
mondo_sensor  *list_end=NULL;
FILE          *fd=NULL;

void sigalrm_handler(int sig)
{
    mondo_sensor  *lsens;
    mondo_action  *tbound;
    double         result;

    for (lsens=list_start; lsens!=NULL; lsens=lsens->next) {
        errno=0;
        result=read_sensor(lsens);

        if (errno!=0) {
            /* We can't rely on the return value of this thing at the
             * moment...just ignore it.
             */
            syslog(LOG_WARNING,
                   "Error reading %s:%s: %s",
                   lsens->chip_label,
                   lsens->feature_label,
                   strerror(errno));

            continue;
        }

        if (lsens->ubound!=NULL && result>lsens->ubound->limit) {
            /* Uh-oh, a sensor has exceeded its limit... */
            if (lsens->throttle>0) {
                /* We should observe some alarm throttling. */
                if (lsens->ticks>0) {
                    /* A throttle countdown is in progress.  Don't actually
                     * act, just decrement the countdown.
                     */
                    lsens->ticks--;
                    continue;
                }

                /* Make sure the throttle countdown gets started. */
                lsens->ticks=lsens->throttle;
            }

            if (lsens->grace>0) {
                /* We should observe a grace period. */
                if (lsens->grace_ticks>0) {
                    /* A grace countdown is in progress.  Don't actually
                     * act, just decrement the countdown.
                     */
                    syslog(LOG_CRIT,
                           "GRACE WARNING: %s:%s exceeded upper limit %.2f at %.2f",
                           lsens->chip_label,
                           lsens->feature_label,
                           lsens->ubound->limit,
                           result);

                    lsens->grace_ticks--;
                    continue;
                }

                /*
                 * The grace countdown doesn't get reset until the sensor
                 * leaves alarm state. 
                 */
                /* lsens->grace_ticks=lsens->grace; */
            }

            syslog(LOG_CRIT,
                   "WARNING! %s:%s exceeded upper limit %.2f at %.2f",
                   lsens->chip_label,
                   lsens->feature_label,
                   lsens->ubound->limit,
                   result);

            for (tbound=lsens->ubound;
                 tbound!=NULL && result>tbound->limit;
                 tbound=tbound->next) {
                kt_spawnsys(tbound->syscmd);
            }
        }
        else if (lsens->lbound!=NULL && result<lsens->lbound->limit) {
            /* Uh-oh, a sensor has gone below its limit... */
            if (lsens->throttle>0) {
                /* We should observe some alarm throttling. */
                if (lsens->ticks>0) {
                    /* A throttle countdown is in progress.  Don't actually
                     * act, just decrement the countdown.
                     */
                    lsens->ticks--;
                    continue;
                }

                /* Make sure the throttle countdown gets started. */
                lsens->ticks=lsens->throttle;
            }

            if (lsens->grace>0) {
                /* We should observe a grace period. */
                if (lsens->grace_ticks>0) {
                    /* A grace countdown is in progress.  Don't actually
                     * act, just decrement the countdown.
                     */
                    syslog(LOG_CRIT,
                           "GRACE WARNING: %s:%s dropped below lower limit %.2f at %.2f",
                           lsens->chip_label,
                           lsens->feature_label,
                           lsens->lbound->limit,
                           result);

                    lsens->grace_ticks--;
                    continue;
                }

                /*
                 * The grace countdown doesn't get reset until the sensor
                 * leaves alarm state. 
                 */
                /* lsens->grace_ticks=lsens->grace; */
            }

            syslog(LOG_CRIT,
                   "WARNING! %s:%s dropped below lower limit %.2f at %.2f",
                   lsens->chip_label,
                   lsens->feature_label,
                   lsens->lbound->limit,
                   result);

            for (tbound=lsens->lbound;
                 tbound!=NULL && result<tbound->limit;
                 tbound=tbound->next) {
                kt_spawnsys(tbound->syscmd);
            }
        }
        else if (lsens->log) {
            if (lsens->throttle>0) {
                /* We should observe some alarm throttling. */
                if (lsens->log_ticks>0) {
                    /* A throttle countdown is in progress.  Don't actually
                     * act, just decrement the countdown.
                     */
                    lsens->log_ticks--;
                    continue;
                }

                /* Make sure the throttle countdown gets started. */
                lsens->log_ticks=lsens->throttle;
            }

            syslog(LOG_INFO,
                   "%s:%s is at %.2f",
                   lsens->chip_label,
                   lsens->feature_label,
                   result);
        }
        else {
            /* This sensor isn't in an alarm state; there's nothing to log or
             * do at this point.  Just make sure throttle and grace counters
             * get reset appropriately.
             */
            lsens->ticks=0;
            lsens->grace_ticks=lsens->grace;
        }
    }
}

int init_lm_api(const char *fname)
{
    if (fname==NULL) {
        errno=EINVAL;
        return -1;
    }

    if (fd!=NULL) {
        errno=EAGAIN;
        return -1;
    }

    fd=fopen(fname, "r");

    if (fd==NULL) {
        return -1;
    }

    return (sensors_init(fd));
}

int shutdown_lm_api()
{
    if (fd==NULL) {
        errno=EBADF;
        return -1;
    }

    sensors_cleanup();
    fclose(fd);
    fd=NULL;
    free_sensor_list();
    return 0;
}

void set_throttle_option(int throttle)
{
    mondo_sensor  *lsens;

    for (lsens=list_start; lsens!=NULL; lsens=lsens->next) {
        if (lsens->throttle<0) {
            lsens->throttle=throttle;
        }
    }
}

void set_grace_option(int grace)
{
    mondo_sensor  *lsens;

    for (lsens=list_start; lsens!=NULL; lsens=lsens->next) {
        if (lsens->grace==0) {
            lsens->grace_ticks=lsens->grace=grace;
        }
    }
}

void set_log_option(int log)
{
    mondo_sensor  *lsens;

    for (lsens=list_start; lsens!=NULL; lsens=lsens->next) {
        if (lsens->log<0) {
            lsens->log=log;
        }
    }
}

void free_sensor_list()
{
    mondo_sensor  *lsens;

    while (list_start!=NULL) {
        lsens=list_start;
        list_start=lsens->next;

        if (lsens->ubound!=NULL) {
            free_alist(lsens->ubound);
        }

        if (lsens->lbound!=NULL) {
            free_alist(lsens->lbound);
        }

        if (lsens->chip_label!=NULL) {
            free(lsens->chip_label);
        }

        if (lsens->feature_label!=NULL) {
            free(lsens->feature_label);
        }

        free(lsens);
    }

    list_start=list_end=NULL;
}

mondo_sensor *locate_sensor(const char *sensor_name)
{
    mondo_sensor                *ret;
    const char                  *ctmp;
    const sensors_feature_data  *sdat;
    int                          nr1 = 0;
    int                          nr2 = 0;
    int                          label_len;

    if (fd==NULL) {
        errno=EPERM;
        return NULL;
    }

    ctmp=strchr(sensor_name, ':');

    if (ctmp==NULL || strchr(ctmp+1, ':')!=NULL) {
        errno=EINVAL;
        return NULL;
    }

    ret=(mondo_sensor *) malloc(sizeof(mondo_sensor));

    if (ret==NULL) {
        return NULL;
    }

    label_len=ctmp-sensor_name+1;
    ret->chip_label=(char *) malloc(label_len);

    if (ret->chip_label==NULL) {
        goto fail_ret_alloced;
    }

    ret->feature_label=strdup(ctmp+1);

    if (ret->feature_label==NULL) {
        goto fail_chip_alloced;
    }

    memcpy(ret->chip_label, sensor_name, label_len-1);
    ret->chip_label[label_len-1]='\0';

    /* Now see about locating this particular sensor feature. */
    if (sensors_parse_chip_name(ret->chip_label, &(ret->chip_id))<0) {
        /* Can't locate this particular chip? */
        errno=ENODEV;
        goto fail_feature_alloced;
    }

    /* Now search this chip's features for the one we want. */
    for ( ; ; ) {
        char *lab;

        sdat=sensors_get_all_features(ret->chip_id, &nr1, &nr2);

        if (sdat==NULL) {
            errno=ENOSYS;
            goto fail_feature_alloced;
        }

        if (sensors_get_label(ret->chip_id, sdat->number, &lab) == 0
          && strcmp(ret->feature_label, lab) == 0) {
            /* We found it. */
            free(lab);
            break;
        }

        free(lab);
    }

    ret->feature_id=sdat->number;
    ret->lbound=ret->ubound=NULL;
    ret->grace=0;
    ret->throttle=ret->log=ret->ticks=ret->log_ticks=ret->grace_ticks=-1;
    ret->next=NULL;

    if (list_start==NULL) {
        /* This will be the first in the list. */
        list_start=list_end=ret;
    }
    else {
        list_end->next=ret;
        list_end=ret;
    }

    return ret;

    fail_feature_alloced:
        free(ret->feature_label);
    fail_chip_alloced:
        free(ret->chip_label);
    fail_ret_alloced:
        free(ret);
        return NULL;
}

double read_sensor(const mondo_sensor *lsens)
{
    int     result;
    double  ret;

    result=sensors_get_feature(lsens->chip_id, lsens->feature_id, &ret);

    if (result<0) {
        if (errno==0) {
            errno=EAGAIN;
        }

        return 0.0;
    }

    return ret;
}
