/***************************************************************************
                 mondo.c  -  the place where everything starts
                             -------------------
    begin                : Wed Dec 12 07:05:40 CST 2001
    copyright            : (C) 2001 by Kelledin
    email                : kelledin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of version 2 of the GNU General Public License as  *
 *   published by the Free Software Foundation.                            *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "mondo.h"

void usage(const char *cmd, FILE *fd);

int main(int argc, char **argv)
{
    int          opt_char;
    int          daemonize    = 1;
    const char  *sensors_file = NULL;
    const char  *opt_string   = "c:s:dhlV";
    char        *cdir;
    char        *tstr;

    do {
        opt_char=getopt(argc, argv, opt_string);

        switch(opt_char) {
        case -1:
            break;

        case 'd':
            daemonize=0;
            break;

        case 'c':
            if (conf_file!=NULL) {
                fprintf(stderr, "FATAL: '-c' option specified twice!\n");
                goto fail_cfile_alloced;
            }

            if (*optarg=='/') {
                /* Good--all we have to do is strdup this bad boy. */
                conf_file=strdup(optarg);
            }
            else {
                /* It's a relative path...we need to do some extra path
                 * construction.
                 */
                cdir=get_current_dir_name();

                if (cdir==NULL) {
                    fprintf(stderr, "FATAL: malloc error: %s\n",
                            strerror(errno));
                    return -1;
                }

                tstr=(char *) malloc(strlen(cdir)+strlen(optarg)+2);

                if (tstr==NULL) {
                    fprintf(stderr, "FATAL: malloc error: %s\n",
                            strerror(errno));
                    free(cdir);
                    return -1;
                }

                strcpy(tstr, cdir);

                if (tstr[strlen(tstr)-1]!='/') {
                    strcat(tstr, "/");
                }

                strcat(tstr, optarg);
                conf_file=tstr;
                free(cdir);
            }

            break;

        case 's':
            if (sensors_file!=NULL) {
                fprintf(stderr, "FATAL: '-s' option specified twice!\n");
                goto fail_cfile_alloced;
            }

            sensors_file=optarg;
            break;

        case 'h':
            usage(*argv, stdout);
            goto exit_cfile_alloced;

        case 'l':
            dolock=0;
            break;

        case 'V':
            printf("System Monitor Daemon v%s\n\n", MONDO_VERSION);
            goto exit_cfile_alloced;

        case '?':
        default:
            usage(*argv, stderr);
            goto fail_cfile_alloced;
        }
    } while (opt_char!=-1);

    if (conf_file==NULL) {
        conf_file=strdup(MONDO_CONF_FILE);

        if (conf_file==NULL) {
            fprintf(stderr, "FATAL: malloc error: %s\n", strerror(errno));
            return -1;
        }
    }

    if (sensors_file==NULL) {
        sensors_file=SENSORS_CONF_FILE;
    }

    if (init_lm_api(sensors_file)<0) {
        fprintf(stderr, "FATAL: initializing lm_sensors: %s\n",
                strerror(errno));

        goto fail_cfile_alloced;
    }

    /* Open the syslog and log a startup message. */
    openlog("mondo", LOG_CONS|LOG_PID, LOG_DAEMON);
    syslog(LOG_INFO, "System Monitor v%s started.", MONDO_VERSION);

    /* Set up the signal handlers and read mondo.conf. */
    if (set_signal_handlers()<0 || read_conf()<0) {
        shutdown_lm_api();
        goto fail_cfile_alloced;
    }

    if (daemonize) {
        /* Go into daemon mode. */
        if (daemon(0, 0)<0) {
            fprintf(stderr, "FATAL: trying to daemonize: %s\n",
                    strerror(errno));

            shutdown_lm_api();
            goto fail_cfile_alloced;
        }

        if (dolock==1 && lock_pidfile()<0) {
            shutdown_lm_api();
            goto fail_cfile_alloced;
        }
    }

    /* Start the alarm ticker. */
    if (set_alrm_timer(mondo_delay)<0) {
        syslog(LOG_ERR, "setting alarm timer: %s", strerror(errno));
        syslog(LOG_ERR, "exiting due to previous errors.");

        if (dolock) {
            unlock_pidfile();
        }

        shutdown_lm_api();
        goto fail_cfile_alloced;
    }

    /* Go into a pause loop. */
    for ( ; ; ) {
        pause();
    }

    exit_cfile_alloced:
        if (conf_file!=NULL) {
            free(conf_file);
        }

        return 0;

    fail_cfile_alloced:
        if (conf_file!=NULL) {
            free(conf_file);
        }

        return -1;
}

void usage(const char *cmd, FILE *fd)
{
    if (fd==NULL) {
        fd=stdout;
    }

    fprintf(fd,
"USAGE: %s [-dhV] [-c config_file] [-s sensors_file]\n"
"\n"
"  -c config_file  : specify a file to read configuration info from\n"
"                    (default is %s)\n"
"  -s sensors_file : specify the location of lm_sensors sensors.conf\n"
"                    (default is %s)\n"
"  -d              : do not daemonize\n"
"  -h              : print out help (this screen)\n"
"  -l              : do not create/lock a pidfile.\n"
"  -V              : print out version.\n\n",
            cmd, MONDO_CONF_FILE, SENSORS_CONF_FILE);
}
