Summary:       Mondo health monitor
Name:          mondo
Version:       0.9
Release:       1
Serial:        20020911
Copyright:     GPL
Group:         System Environment/Daemons
Source:        ftp://ftp.sourceforge.net/pub/sourceforge/%{name}-daemon/%{name}-%{version}.tar.gz
BuildRoot:     %{_tmppath}/%{name}-%{version}-%{release}-%(id -u)-%(id -g)-build
BuildRequires: lm_sensors >= 2.6.0
BuildRequires: glibc
BuildRequires: fileutils
BuildRequires: textutils
BuildRequires: sh-utils
BuildRequires: make
BuildRequires: bash
BuildRequires: tar
BuildRequires: gzip
BuildRequires: sed
BuildRequires: grep
BuildRequires: gcc

%description
Mondo is a system health monitor daemon for Linux, capable of running specific commands whenever alarm conditions get tripped.  Mondo can do things like send e-mails and shut your system down if a fan fails, voltage rails start to go haywire, or your system just seems warmer than it should be.

Install mondo if you would like an automated system monitor facility.

%prep

%setup -n %{name}-%{version}
# =========================================================================== #
# SETUP SECTION                                                               #
# =========================================================================== #

# Set the default umask.
umask 022

# Clean up BuildRoot directory.
if [ "$RPM_BUILD_ROOT" = "/" ]; then
    # We can't use / as a build root!
    echo 'ERROR: Use of "/" as build root is not supported!' >& 2
    exit 1
fi

rm -Rf "$RPM_BUILD_ROOT"

# Recreate the standard base directories in the build root.
mkdir -p "$RPM_BUILD_ROOT"
cd "$RPM_BUILD_ROOT"

mkdir -p bin boot dev/pts etc/opt home lib mnt proc root sbin tmp var opt usr
cd usr
mkdir -p bin etc include lib libexec sbin share src var
ln -sf share/man man
ln -sf share/doc doc
ln -sf share/info info
ln -sf ../tmp tmp
cd share
mkdir -p dict doc info locale man nls misc terminfo zoneinfo
cd man
mkdir -p man{1,2,3,4,5,6,7,8}
cd ../../../var
mkdir -p lock log mail run spool tmp opt cache lib/misc local
cd ../opt
mkdir -p bin doc include info lib man
cd ../etc
mkdir -p syscfg rc.d init.d rc{0,1,2,3,4,5,6}.d
ln -sf rc1.d rcS.d
ln -sf rc1.d rcs.d
cd ..
chmod 1777 tmp var/tmp

%build
# =========================================================================== #
# BUILD SECTION                                                               #
# =========================================================================== #

# Set the default umask.
umask 022

# Get the number of CPUs...
if [ -f /proc/cpuinfo ] && [ -r /proc/cpuinfo ]; then
    # Get the number of CPUs from /proc/cpuinfo...
    MONDO_NCPUS=`grep -c "^processor.*: [0-9]*$" /proc/cpuinfo`
else
    # D'oh, assume we only have one CPU...
    MONDO_NCPUS=1
fi

# Now figure out the number of jobs to have make spawn...(num_cpus+1) seems
# to be the sweet spot for most machines.
MONDO_NJOBS=$(($MONDO_NCPUS+1))

# Export RPM_OPT_FLAGS for the build script's use.
export CFLAGS=$RPM_OPT_FLAGS
export CXXFLAGS=$RPM_OPT_FLAGS

# Perform build steps, taking target CPU and architecture into account.
./configure \
            --prefix=/usr \
            --sysconfdir=/etc \
            --localstatedir=/var \
            --host=%{_target_platform}-gnu \
            --target=%{_target_platform}-gnu \
            --build=%{_target_platform}-gnu

make \
     -j $MONDO_NJOBS

%install
# =========================================================================== #
# INSTALL SECTION                                                             #
# =========================================================================== #

# Set the default umask.
umask 022

# Get the number of CPUs...
if [ -f /proc/cpuinfo ] && [ -r /proc/cpuinfo ]; then
    # Get the number of CPUs from /proc/cpuinfo...
    MONDO_NCPUS=`grep -c "^processor.*: [0-9]*$" /proc/cpuinfo`
else
    # D'oh, assume we only have one CPU...
    MONDO_NCPUS=1
fi

# Now figure out the number of jobs to have make spawn...(num_cpus+1) seems
# to be the sweet spot for most machines.
MONDO_NJOBS=$(($MONDO_NCPUS+1))

# Do the install, taking $RPM_BUILD_ROOT into account.
make \
     -j $MONDO_NJOBS \
     DESTDIR="$RPM_BUILD_ROOT" \
     install-strip

chown -R root:root *

# =========================================================================== #
# FILES SECTION                                                               #
# =========================================================================== #
%files
%defattr(-, root, root)
%doc AUTHORS
%doc COPYING
%doc FAQ
%doc ChangeLog
%doc INSTALL
%doc README
%doc README.CPUS
%doc README.grace
%doc TODO
%attr(0644, root, root) %verify(not md5 size mtime link) %config /etc/mondo.conf
%attr(0750, root, root) /etc/init.d/mondo
%attr(0755, root, root) /usr/sbin/mondo
%attr(0755, root, root) /usr/sbin/mondo-setup
%attr(0644, root, root) /usr/man/man5/mondo.conf.5*
%attr(0644, root, root) /usr/man/man8/mondo.8*
%attr(0644, root, root) /usr/man/man8/mondo-setup.8*

%clean
# =========================================================================== #
# CLEANUP SECTION                                                             #
# =========================================================================== #

# Clean up behind ourselves...
if [ ! "$RPM_BUILD_ROOT" = "/" ]; then
    rm -Rf "$RPM_BUILD_ROOT"
fi

rm -Rf "$RPM_BUILD_DIR/%{name}-%{version}"
exit 0

# =========================================================================== #
# INSTALL/UNINSTALL/UPGRADE SCRIPTLETS                                        #
# =========================================================================== #
%post
cvxcmd() {
    # First, clear the bash hash.
    hash -r

    # Now determine what type of command it is.
    case `type -t "$1"` in
        file)
            # It's a file.  What is its full path?
            CVXPATH=`type -p "$1"`

            # Is it executable?
            if [ "x$CVXPATH" = "x" ] || [ ! -x "$CVXPATH" ]; then
                return 1
            fi

            echo "$CVXPATH"
            ;;

        builtin)
            # It's a shell built-in.  That's all we need to know.
            echo "$1"
            ;;

        alias)
            # Alias?  We don't want a steenking alias...screw aliases, we want
            # a full path.
            CVXPATH=`type -ap "$1"`

            # Is it executable?
            if [ "x$CVXPATH" = "x" ] || [ ! -x "$CVXPATH" ]; then
                return 1
            fi

            echo "$CVXPATH"
            ;;

        *)
            # This is some type I'm not designed to understand...
            return 1
            ;;
    esac

    return 0
}

umask 022
MAKEWHATIS=`cvxcmd makewhatis`

if [ ! "x$MAKEWHATIS" = "x" ]; then
    $MAKEWHATIS -u /usr/man
fi

if [ -x "/etc/init.d/mondo" ] &&
   [ -r "/etc/init.d/mondo" ] &&
   [ -f "/etc/init.d/mondo" ]; then
    if [ "$1" = "1" ]; then
        if [ -x "/usr/lib/lsb/install_initd" ] &&
           [ -r "/usr/lib/lsb/install_initd" ] &&
           [ -f "/usr/lib/lsb/install_initd" ]; then
            /usr/lib/lsb/install_initd /etc/init.d/mondo
        fi
    else
        /etc/init.d/mondo status >& /dev/null

        if [ "$?" = "0" ]; then
            /etc/init.d/mondo restart
        fi
    fi
fi

exit 0

%preun
umask 022

if [ "$1" = "0" ] &&
   [ -x "/etc/init.d/mondo" ] &&
   [ -r "/etc/init.d/mondo" ] &&
   [ -f "/etc/init.d/mondo" ]; then
    /etc/init.d/mondo status >& /dev/null

    if [ "$?" = "0" ]; then
        /etc/init.d/mondo stop
    fi

    if [ -x "/usr/lib/lsb/remove_initd" ] &&
       [ -r "/usr/lib/lsb/remove_initd" ] &&
       [ -f "/usr/lib/lsb/remove_initd" ]; then
        /usr/lib/lsb/remove_initd /etc/init.d/mondo
    fi
fi

exit 0

%postun
cvxcmd() {
    # First, clear the bash hash.
    hash -r

    # Now determine what type of command it is.
    case `type -t "$1"` in
        file)
            # It's a file.  What is its full path?
            CVXPATH=`type -p "$1"`

            # Is it executable?
            if [ "x$CVXPATH" = "x" ] || [ ! -x "$CVXPATH" ]; then
                return 1
            fi

            echo "$CVXPATH"
            ;;

        builtin)
            # It's a shell built-in.  That's all we need to know.
            echo "$1"
            ;;

        alias)
            # Alias?  We don't want a steenking alias...screw aliases, we want
            # a full path.
            CVXPATH=`type -ap "$1"`

            # Is it executable?
            if [ "x$CVXPATH" = "x" ] || [ ! -x "$CVXPATH" ]; then
                return 1
            fi

            echo "$CVXPATH"
            ;;

        *)
            # This is some type I'm not designed to understand...
            return 1
            ;;
    esac

    return 0
}

umask 022
MAKEWHATIS=`cvxcmd makewhatis`

if [ ! "x$MAKEWHATIS" = "x" ]; then
    $MAKEWHATIS -u /usr/man
fi

exit 0

# =========================================================================== #
# PACKAGE CHANGELOG                                                           #
# =========================================================================== #
%changelog
* Wed Sep 11 2002 Kelledin <kelledin@users.sourceforge.net>
- updated %post, %preun, and %postun scriptlets.
- added %defattr tag.
- added %doc README.grace.
- updated to version 0.9 (release 0.9-1)

* Sat Aug 24 2002 Kelledin <kelledin@users.sourceforge.net>
- added extra %doc files.
- updated to version 0.8 (release 0.8-1)

* Tue Jul 30 2002 Kelledin <kelledin@users.sourceforge.net>
- updated to version 0.7 (release 0.7-1)

* Sat Jun 22 2002 Kelledin <kelledin@users.sourceforge.net>
- updated to version 0.6a (release 0.6a-1)

* Wed Apr 10 2002 Kelledin <kelledin@users.sourceforge.net>
- miscellaneous spec updates.
- updated %post and %postun scriptlets.
- updated to version 0.6 (release 0.6-1)

* Thu Jan 17 2002 Kelledin <kelledin@users.sourceforge.net>
- updated %post and %postun scriptlets.
- updated to version 0.5 (release 0.5-1)

* Tue Jan  1 2002 Kelledin <kelledin@users.sourceforge.net>
- updated to version 0.4 (release 0.4-1)

* Thu Dec 27 2001 Kelledin <kelledin@users.sourceforge.net>
- updated to version 0.3 (release 0.3-1)

* Tue Dec 18 2001 Kelledin <kelledin@users.sourceforge.net>
- updated to version 0.2 (release 0.2-1)

* Mon Dec 17 2001 Kelledin <kelledin@users.sourceforge.net>
- packaged in RPM format (release 0.1-1)
